﻿-----------------------------------------------------------------------------
--  Tabard Addict - TabardAddict.lua
--
--  This file contains the majority of the functionality for the addon.
--
--  functions:
--			local function DressUpTexturePath(raceFileName)
--			function SetTADressUpBackground()
--			local function TabardAddict_CheckPlayerFaction()
--			local function TabardAddict_GetStandingTxt(standingID)
--			local function TabardAddict_BuildTooltipData()
--			local function TabardAddict_CheckReputationRequirement(itemID)
--			local function TabardAddict_CheckQuestRequirement(itemID)
--			local function TabardAddict_CheckAchievementRequirement(itemID)
--			local function TabardAddict_CheckCurrencyRequirement(itemID)
--			local function TabardAddict_CheckDropRequirement(itemID)
--			local function TabardAddict_CheckSpecialRequirement(itemID)
--			local function TabardAddict_PopulateDisplayTable()
--			local function TabardAddict_PopulateSuggestionsTable()
--			local function TabardAddict_SortDisplayTable()
--			local function TabardAddict_SortSuggestionsTable()
--			local function TabardAddict_CleanConfigVars()
--			function TabardAddict_OnEvent(self, event, ...)
--			function TabardAddict_GetServerData()
--			function TabardAddictScrollBar_Update()
--			function TabardAddictScrollBar2_Update()
--			function TabardAddictEntryButton_OnEnter(line)
--			function TabardAddictSuggButton_OnEnter(line)
--			function TabardAddictEntryButton_OnLeave()
--			function TabardAddictSuggButton_OnLeave()
--			function TabardAddictEntryButton_OnUpdate(line)
--			function TabardAddictSuggButton_OnUpdate(line)
--			function TabardAddictEntryButton_OnModifiedClick(line)
--			function TabardAddictSuggButton_OnModifiedClick(line)
--			function TabardAddictTab_OnClick(self, tabID)
--			function TabardAddictEntryButton_OnClick(self, line, section)
--			function TabardAddict_InfoFrameShowEquipment_OnClick(self)
--			function TabardAddictLauncher_OnClick(self)
--			function TabardAddictSearchToggle_OnClick(self)
--			function TabardAddictSearchClear_OnClick(self)
--			function TabardAddict_OnShow(self)
--			function TabardAddictLauncher_OnLoad(self)
--			function TabardAddict_OnHide(self)
--			function TabardAddictShowTabards()
--			function TabardAddictShowSuggestions()
--			function TabardAddictHideTabards()
--			function TabardAddictHideSuggestions()
--			function TabardAddict_SetConfigVariables(self)
--			function TabardAddict_ConfigOnEnter(self)
--			function TabardAddict_TotalText_OnEnter(self)
--			function TabardAddict_OnTextChanged(self, userInput)
--			local function OnTooltipSetItem(tooltip, ...)
--			local function OnTooltipCleared(tooltip, ...)
--			local function ta_GetDDConfigValue(param)
--			local function ta_SetDDConfigValue(self, arg1, arg2, checked)
--			local function ta_CheckAllExpansionsDDConfigValue(self, arg1, arg2)
--			function TabardAddict_FilterMenu_OnLoad()
--			function TabardAddict_FilterMenuButton_OnClick(self)
--			function TabardAddict_ConfigExpansionsButton_OnClick(self)
-- 			function TabardAddict_ConfigExpansions_OnLoad()
--			function TabardAddict_OnLoad(self)
--
--  (c) 2011-2024 gmz323(Greg)
-----------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- Constant and Local declarations
-----------------------------------------------------------------------------
local TABARD_ADDICT_VERSION = "2.69"
local TABARD_ADDICT_DATA_ROWS = 217

local TabardAddictServerData = {} -- equip data from server
local TabardAddictServerData_tooltiptable = {} -- table for tooltip display
local TabardAddictDoServerQuery
local TabardAddictUpdateSuggestTable
local TabardAddictUpdateDisplayTable

local CurrentTabardEntries_itemID = {}
local CurrentTabardEntries_itemLink = {}
local CurrentTabardSuggest_itemID = {}
local CurrentTabardSuggest_itemLink = {}
local InfoFrameCurrent_itemID = 0
local InfoFrameCurrent_itemLink = ""

local tempDisplayData = {} -- temp table built to show subset of data
local tempDisplayData_rowcount
local tempSuggestData = {} -- temp table built to show subset of data
local tempSuggestData_rowcount

local tempAchievementTooltipData = {} -- table holding achievement tooltip data
local tabardsUnlocked = 0
local taNameFilter = ""
local taLineAdded = false
local taPlayerFaction = 0 -- 1=alliance, 2=horde, 3=neutral
local taPlayerLevel = 0
local taSuggestString = ""

-- Localized Strings
local L = TabardAddictLocalization

BACKDROP_TABARDADDICT_INFOFRAME = {
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true,
	tileSize = 32,
	edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 },
};

-- Slash commands
SLASH_TABARDADDICT1 = "/ta"
SLASH_TABARDADDICT2 = "/tabardaddict"
BINDING_HEADER_TABARDADDICT = L.ADDON_TITLE
BINDING_NAME_TABARDADDICT_TOGGLE = L.LAUNCHER_TT


SlashCmdList["TABARDADDICT"] = function(msg, editbox)
	if frameTabardAddict:IsShown() then
		frameTabardAddict:Hide()
	else
		frameTabardAddict:Show()
	end
end

-- Filter Menu Text
local FilterMenuInfo = {
	{ ["text"] = TabardAddictLocalization["BEEN_EQUIPPED"] .. "|cff00FF00".. TabardAddictLocalization["YES"] .. "|r", ["arg1"] = "CB_FILTER_EQYES",  },
	{ ["text"] = TabardAddictLocalization["BEEN_EQUIPPED"] .."|cffFF0000" .. TabardAddictLocalization["NO"] .. "|r", ["arg1"] = "CB_FILTER_EQNO",  },
	{ ["text"] = TabardAddictLocalization["BEEN_EQUIPPED"] .. "|cff808080" .. TabardAddictLocalization["NOT_TRACKED"] .. "|r", ["arg1"] = "CB_FILTER_NTBG",  },
}

-- Debugging
local debugMode = false  

-- copied from ModelFrames.lua
local playerRaceSex
if ( not IsOnGlueScreen() ) then
	local _
	_, playerRaceSex = UnitRace("player")
	if ( UnitSex("player") == 2 ) then
		playerRaceSex = playerRaceSex.."Male"
	else
		playerRaceSex = playerRaceSex.."Female"
	end
end
	

-----------------------------------------------------------------------------
-- Returns a texture file path based on filename.
--
-- @param raceFileName  fileName valued returned by a UnitRace() lookup.
-- @return 				texture file path.
-----------------------------------------------------------------------------
local function DressUpTexturePath(raceFileName)
    -- Orc texture as default
    if ( not raceFileName ) then
    	raceFileName = "Orc"
    end
    -- texture file path
    return "Interface\\DressUpFrame\\DressUpBackground-"..raceFileName
end


-----------------------------------------------------------------------------
-- Sets the background for the dressup window based on the race.
-----------------------------------------------------------------------------
function SetTADressUpBackground()
	local race, fileName = UnitRace("player")
	local texture = DressUpTexturePath(fileName)
	TADressUpBackgroundTop:SetTexture(texture..1)
end


-----------------------------------------------------------------------------
-- Sets global variable taPlayerFaction.
-----------------------------------------------------------------------------
local function TabardAddict_CheckPlayerFaction()

	-- set alliance or horde background
	local englishFaction = UnitFactionGroup("player")
	if ( englishFaction == "Horde" ) then  -- horde
		TabardAddictBGFaction:Show()
		TabardAddictBGFaction2:Hide()
		TabardAddictBGFactionSugg:Show()
		TabardAddictBGFactionSugg2:Hide()
		taPlayerFaction = 2
	elseif ( englishFaction == "Alliance" ) then
		TabardAddictBGFaction:Hide()
		TabardAddictBGFaction2:Show()
		TabardAddictBGFactionSugg:Hide()
		TabardAddictBGFactionSugg2:Show()
		taPlayerFaction = 1
	else  -- Neutral faction - Pandaren 1-10
		TabardAddictBGFaction:Hide()
		TabardAddictBGFaction2:Hide()
		TabardAddictBGFactionSugg:Hide()
		TabardAddictBGFactionSugg2:Hide()
		taPlayerFaction = 3
	end

end


-----------------------------------------------------------------------------
-- Returns localized string for the standing (Exalted, Revered, Honored, etc..
-- based on locale and GlobalStrings file).
--
-- @param standingID  	Identifies a level of reputation.
-- @return 				Localized string for the standingID and gender.
-----------------------------------------------------------------------------
local function TabardAddict_GetStandingTxt(standingID)
	local standingTxt
	
	local gender = UnitSex("player")

	if (standingID >= 1 and standingID <= 8) then
		-- female gender - important for non-us locale
		if (gender == 3 ) then 
			standingTxt = _G['FACTION_STANDING_LABEL'..standingID..'_FEMALE']
		else
			standingTxt = _G['FACTION_STANDING_LABEL'..standingID]
		end
	else
		standingTxt = ""
	end
	
	return standingTxt
end


-----------------------------------------------------------------------------
-- Loops through the series of tabard collecting achievements and determines 
-- what has been completed. Builds tooltip data for display.
-----------------------------------------------------------------------------
local function TabardAddict_BuildTooltipData()

	-- 621 - first tabard achievement
	local achievementID = 621
	local i = 1

	if (debugMode) then
		print("BuildTooltipData")
	end
	
	while (achievementID ~= nil)
	do
		if (debugMode) then
			print("achievementID: " .. tostring(achievementID))
		end
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch, wasEarnedByMe, earnedBy = GetAchievementInfo(achievementID)
		if (id ~= nil) then
			tempAchievementTooltipData[i] = {}
			tempAchievementTooltipData[i][1] = completed
			tempAchievementTooltipData[i][2] = name
			tempAchievementTooltipData[i][3] = earnedBy
			if (completed) then
				tempAchievementTooltipData[i][4] = string.format(SHORTDATE, day, month, year)
			end
			i = i + 1
			local nextID, completed = GetNextAchievement(id)
			achievementID = nextID
		else
			achievementID = nil
		end
	end

end


-----------------------------------------------------------------------------
-- Checks if player is above required faction level to equip specific 
-- tabard.
--
-- @param itemID  		id of tabard to check.
-- @return 				1 (player can equip) or 0 (player can NOT equip).
-----------------------------------------------------------------------------
local function TabardAddict_CheckReputationRequirement(itemID)
	-- return 1(suggest) or 0(don't suggest)
	
	-- get reputation for this tabards faction - faction id in taTabardReqmtData[itemID][3]
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfoByID(taTabardReqmtData[itemID][3])
	
	-- handle bad faction lookup
	if (name == nil) then
		return 0
	end
	
	-- compare against required level - level in [4]
	if (standingID >= taTabardReqmtData[itemID][4]) then
		taSuggestString = string.format(FACTION_STANDING_CHANGED, TabardAddict_GetStandingTxt(taTabardReqmtData[itemID][4]), name)
		return 1
	else
		return 0
	end
end


-----------------------------------------------------------------------------
-- Checks if player meets requirements to get quest based tabard.
--
-- @param itemID  		id of tabard to check.
-- @return 				1 (player meets req) or 0 (player does NOT meet req).
-----------------------------------------------------------------------------
local function TabardAddict_CheckQuestRequirement(itemID)
	
	-- get quest name from questid - taTabardReqmtData[itemID][3]
	-- get quest level from - taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player")
	
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		if (1) then -- quest config
			taSuggestString = L.SUGG_QUEST_TXT .. " " .. taTabardReqmtData[itemID][3]
			return 1
		else
			return 0
		end
	end
	return 0
end


-----------------------------------------------------------------------------
-- Checks if player meets requirements to get achievement based tabard.
--
-- @param itemID  		id of tabard to check.
-- @return 				1 (player meets req) or 0 (player does NOT meet req).
-----------------------------------------------------------------------------
local function TabardAddict_CheckAchievementRequirement(itemID)
	
	-- get achievement name from achievementid - taTabardReqmtData[itemID][3]
	-- get achievement level from taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player")

	-- achievement levels are currently 0 - update!!!
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		-- get achievement name
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch,  wasEarnedByMe, earnedBy = GetAchievementInfo(taTabardReqmtData[itemID][3])
		if (name == nil) then
			return 0
		end
		taSuggestString = L.SUGG_ACHV_TXT .. name
		return 1
	else
		return 0
	end
end


-----------------------------------------------------------------------------
-- Checks if player meets requirements to get currency based tabard.
--
-- @param itemID  		id of tabard to check.
-- @return 				1 (player meets req) or 0 (player does NOT meet req).
-----------------------------------------------------------------------------
local function TabardAddict_CheckCurrencyRequirement(itemID)
	
	-- currency amount and name string - taTabardReqmtData[itemID][3]
	-- required level - taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player")
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		taSuggestString = L.SUGG_CURRENCY_TXT .. taTabardReqmtData[itemID][3]
		return 1
	else
		return 0
	end
end

-----------------------------------------------------------------------------
-- Checks if player meets requirements to get drop based tabard.
--
-- @param itemID  		id of tabard to check.
-- @return 				1 (player meets req) or 0 (player does NOT meet req).
-----------------------------------------------------------------------------
local function TabardAddict_CheckDropRequirement(itemID)
	
	-- mob id taTabardReqmtData[itemID][3]
	-- mob level taTabardReqmtData[itemID][4]
	local playerLevel = UnitLevel("player")
	if (playerLevel >=  taTabardReqmtData[itemID][4]) then
		-- set taSuggestString
		taSuggestString = L.SUGG_DROP_TXT .. taTabardReqmtData[itemID][3] .. " (" .. LEVEL_ABBR .. " " .. taTabardReqmtData[itemID][4] .. ")"
		return 1
	else
		return 0
	end
end


-----------------------------------------------------------------------------
-- Checks if player meets requirements to get special requirement tabard.
--
-- @param itemID  		id of tabard to check.
-- @return 				1 (player meets req) or 0 (player does NOT meet req).
-----------------------------------------------------------------------------
local function TabardAddict_CheckSpecialRequirement(itemID)

	-- handle the midsummer event tabards - 35279, 35280
	-- if one has been equipped dont suggest the other
	if (itemID == 35279) then
		if (TabardAddictServerData[35280] == true) then
			return 0 -- can't equip both tabards
		else
			return 1
		end
	end
	if (itemID == 35280) then
		if (TabardAddictServerData[35279] == true) then
			return 0 -- can't equip both tabards
		else
			return 1
		end
	end
	
	-- handle paladin blood elf
	if (itemID == 25549) then
		local class, classFileName = UnitClass("player")
		local race, raceFileName = UnitRace("player")
		
		if ( (strupper(classFileName) == "PALADIN") and (strupper(raceFileName) == "BLOODELF") ) then
			return 1
		else
			return 0
		end
	end
	
	-- handle draenei only
	if (itemID == 24344) then
		local race, raceFileName = UnitRace("player")
		
		if (strupper(raceFileName) == "DRAENEI") then
			return 1
		else
			return 0
		end
	end

	-- handle undead race tabard (string as Scourge)
	if ((itemID == 209065) or (itemID == 209068)) then
		local race, raceFileName = UnitRace("player")
		
		if (strupper(raceFileName) == "SCOURGE") then
			return 1
		else
			return 0
		end
	end

	-- handle night elf tabard
	if (itemID == 210424) then
		local race, raceFileName = UnitRace("player")
		
		if (strupper(raceFileName) == "NIGHTELF") then
			return 1
		else
			return 0
		end
	end

	-- handle Battle of the Crimson Watch rewards
	-- if one has been equipped dont suggest the other
	if (itemID == 31404) then
		if (TabardAddictServerData[31405] == true) then
			return 0 -- can't equip both tabards
		else
			return 1
		end
	end
	if (itemID == 31405) then
		if (TabardAddictServerData[31404] == true) then
			return 0 -- can't equip both tabards
		else
			return 1
		end
	end

	-- handle Demon Hunter tabard
	if (itemID == 133670) then
		local class, classFileName = UnitClass("player")
		if (strupper(classFileName) == "DEMONHUNTER") then
			return 1
		else
			return 0
		end
	end

	-- handle Lightforged Draenei tabard
	if (itemID == 157756) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "LIGHTFORGEDDRAENEI") then
			return 1
		else
			return 0
		end
	end
	
	-- handle Highmountain Tauren tabard
	if (itemID == 157757) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "HIGHMOUNTAINTAUREN") then
			return 1
		else
			return 0
		end
	end

	-- handle Void Elf tabard
	if (itemID == 157758) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "VOIDELF") then
			return 1
		else
			return 0
		end
	end

	-- handle Nightborne tabard
	if (itemID == 157759) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "NIGHTBORNE") then
			return 1
		else
			return 0
		end
	end

	-- handle Maghar Orc Clan tabard
	if (itemID == 161328) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "MAGHARORC") then
			return 1
		else
			return 0
		end
	end

	-- handle Dark Iron Dwarves tabard
	if (itemID == 161329) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "DARKIRONDWARF") then
			return 1
		else
			return 0
		end
	end	

	-- handle Zandalari Troll tabard
	if (itemID == 165001) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "ZANDALARITROLL") then
			return 1
		else
			return 0
		end
	end	

	-- handle KulTiran Human tabard
	if (itemID == 165010) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "KULTIRAN") then
			return 1
		else
			return 0
		end
	end	

	-- handle Mechagnome
	if (itemID == 174068) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "MECHAGNOME") then
			return 1
		else
			return 0
		end
	end	

	-- handle Vulpera
	if (itemID == 174069) then
		local race, raceFileName = UnitRace("player")
		if (strupper(raceFileName) == "VULPERA") then
			return 1
		else
			return 0
		end
	end	

	return 1 -- not a special requirement tabard
end


-----------------------------------------------------------------------------
-- Populates the global tempDisplayData table as needed.
-----------------------------------------------------------------------------
local function TabardAddict_PopulateDisplayTable()
	
	-- if the displaydata table is nil force an update
	if (tempDisplayData == nil) then
		TabardAddictUpdateDisplayTable = true
	end

	-- Only need to populate this table once - and when server data is rechecked or config change
	if (TabardAddictUpdateDisplayTable == true) then

		local maintableRow, j, displayThisRow
		wipe(tempDisplayData)
		tempDisplayData_rowcount = 0
		TabardAddictUpdateDisplayTable = false

		if (debugMode) then
			print ("Tabard Addict: PopulateDisplayTable() called")
			for index,value in pairs(TabardAddictConfig_v2) do
				print(tostring(index) .. " : " .. tostring(value))
			end
		end
		
		-- This creates a temp table by joining local tabard data and server results
		for maintableRow in ipairs(taTabardData)
		do
			-- determine if this row should be displayed -- by Tabard Type
			if ((taTabardData[maintableRow][5] == 0) and (TabardAddictConfig_v2.CB_Other == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 1) and (TabardAddictConfig_v2.CB_PVP == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 2) and (TabardAddictConfig_v2.CB_Other == true)) then
				displayThisRow = 1	
			elseif ((taTabardData[maintableRow][5] == 3) and (TabardAddictConfig_v2.CB_TCG == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 4) and (TabardAddictConfig_v2.CB_NA == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 6) and (TabardAddictConfig_v2.CB_BCF == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 7) and (TabardAddictConfig_v2.CB_WotLKF == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 9) and (TabardAddictConfig_v2.CB_CF == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 10) and (TabardAddictConfig_v2.CB_MOPF == true)) then
				displayThisRow = 1	
			elseif ((taTabardData[maintableRow][5] == 11) and (TabardAddictConfig_v2.CB_WOD == true)) then
				displayThisRow = 1	
			elseif ((taTabardData[maintableRow][5] == 12) and (TabardAddictConfig_v2.CB_LG == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 13) and (TabardAddictConfig_v2.CB_BFA == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 14) and (TabardAddictConfig_v2.CB_SLANDS == true)) then
				displayThisRow = 1
			elseif ((taTabardData[maintableRow][5] == 15) and (TabardAddictConfig_v2.CB_DFLIGHT == true)) then
				displayThisRow = 1
			else
				displayThisRow = 0
			end
			
			-- determine if this row should be filtered -- Filter out by Alliance/Horde
			if ((taTabardData[maintableRow][6] == 1) and (TabardAddictConfig_v2.CB_AO == false)) then
				displayThisRow = 0
			elseif ((taTabardData[maintableRow][6] == 2) and (TabardAddictConfig_v2.CB_HO == false)) then
				displayThisRow = 0
			end

			-- handle filter drop down
			-- hide/show equipped tabards
			local iID = taTabardData[maintableRow][1]
			if (iID ~= nil) then
				if((TabardAddictServerData[iID] == true) and (TabardAddictConfig_v2.CB_FILTER_EQYES == false)) then
					displayThisRow = 0
				elseif ((TabardAddictServerData[iID] == false) and (TabardAddictConfig_v2.CB_FILTER_EQNO == false)) then
					displayThisRow = 0
				elseif ((TabardAddictServerData[iID] == 'na') and (TabardAddictConfig_v2.CB_FILTER_NTBG == false)) then
					displayThisRow = 0
				end
			end


			-- handle search box
			if ((displayThisRow == 1) and (taNameFilter ~= "")) then
				if( string.find(string.lower(taTabardData[maintableRow][2]),string.lower(taNameFilter)) == nil ) then
					displayThisRow = 0
				end
			end
			
			if (displayThisRow == 1) then
				tempDisplayData_rowcount = tempDisplayData_rowcount + 1
				tempDisplayData[tempDisplayData_rowcount] = {}    -- create a new row
				for j=1,4 do -- copy first 4 column
					tempDisplayData[tempDisplayData_rowcount][j] = taTabardData[maintableRow][j]
				end
				-- column 5 - init equipped bool
				tempDisplayData[tempDisplayData_rowcount][5] = false
				-- column 6 - tabard type
				tempDisplayData[tempDisplayData_rowcount][6] = taTabardData[maintableRow][5]
				-- column 7 - alliance or horde indicator
				tempDisplayData[tempDisplayData_rowcount][7] = taTabardData[maintableRow][6]
			    -- column 8 - sourceID (was criteria ID)
				tempDisplayData[tempDisplayData_rowcount][8] = taTabardData[maintableRow][7]
				
				
				-- set equipped bool from server data
				local iID = taTabardData[maintableRow][1]
				if (iID ~= nil) then
					tempDisplayData[tempDisplayData_rowcount][5] = TabardAddictServerData[iID]
				end
				
			end
	    end
		TabardAddictTab1.taTabardTotalShowing:SetText(string.format(L.TABARDS_SHOWN, tempDisplayData_rowcount))
	end
end


-----------------------------------------------------------------------------
-- Populates the global tempSuggestData table as needed.
-----------------------------------------------------------------------------
local function TabardAddict_PopulateSuggestionsTable()

	-- if the suggest table is nil force an update
	if (tempSuggestData == nil) then
		TabardAddictUpdateSuggestTable = true
	end
	
	-- Only need to populate this table once - and when server date is rechecked or config change
	if (TabardAddictUpdateSuggestTable == true) then
	
		if (debugMode) then
			print("Tabard Addict: PopulateSuggestionsTable() called")
		end
	
		local maintableRow, j, suggestThisRow
		wipe(tempSuggestData)
		tempSuggestData_rowcount = 0
		TabardAddictUpdateSuggestTable = false
		
		-- Check Alliance/Horde/Neutral
		TabardAddict_CheckPlayerFaction()
		
		-- This creates a temp table by joining local tabard data and server results
		for maintableRow in ipairs(taTabardData)
		do
			-- init var
			suggestThisRow = 0
			--get item id
			local iID = taTabardData[maintableRow][1]

			-- check if been equipped
			if (TabardAddictServerData[iID] == false) then
				suggestThisRow = 1
			end
			-- check if currently available 
			if (suggestThisRow == 1) then
				if (taTabardReqmtData[iID][0] ~= 1) then
					suggestThisRow = 0
				end
			end
			-- check alliance(1) and horde(2) factions
			if (suggestThisRow == 1) then
				if (taTabardReqmtData[iID][1] == 1 and taPlayerFaction ~= 1 ) then
					suggestThisRow = 0
				elseif (taTabardReqmtData[iID][1] == 2 and taPlayerFaction ~= 2 ) then
					suggestThisRow = 0
				else
					suggestThisRow = 1
				end
			end
			
			-- init Suggestion String
			taSuggestString = ""
			
			-- check requirements
			if (suggestThisRow == 1) then
				-- check the rep/quest/etc requirements
				if (taTabardReqmtData[iID][2] == 0) then
					suggestThisRow = 1
					taSuggestString = L.SUGG_NOREQ_TXT
				elseif (taTabardReqmtData[iID][2] == 1) then
					-- requires reputation 
					suggestThisRow = TabardAddict_CheckReputationRequirement(iID)
				elseif (taTabardReqmtData[iID][2] == 2 and TabardAddictConfig_v2.CB_SUGG_QST == true) then
					-- requires quest completion
					suggestThisRow = TabardAddict_CheckQuestRequirement(iID)
				elseif (taTabardReqmtData[iID][2] == 3 and TabardAddictConfig_v2.CB_SUGG_ACH == true) then
					-- requires achievement completion
					suggestThisRow = TabardAddict_CheckAchievementRequirement(iID)
				elseif (taTabardReqmtData[iID][2] == 4 and TabardAddictConfig_v2.CB_SUGG_CUR == true) then
					-- requires currency to purchase
					suggestThisRow = TabardAddict_CheckCurrencyRequirement(iID)
				elseif (taTabardReqmtData[iID][2] == 5) then
					-- requires drop from mob
					suggestThisRow = TabardAddict_CheckDropRequirement(iID)
				else
					suggestThisRow = 0
				end
			end
			
			-- handle special cases - class/race tabards, event tabards, etc...
			if (suggestThisRow == 1) then
				if (taTabardReqmtSpecial[iID]) then
					suggestThisRow = TabardAddict_CheckSpecialRequirement(iID)
				end
			end
			
			-- add to table if passed all the checks
			if (suggestThisRow == 1) then
				tempSuggestData_rowcount = tempSuggestData_rowcount + 1
				tempSuggestData[tempSuggestData_rowcount] = {}    -- create a new row
				for j=1,4 do -- copy first 4 column
					tempSuggestData[tempSuggestData_rowcount][j] = taTabardData[maintableRow][j]
				end
				-- column 5 - been equipped - must be false
				tempSuggestData[tempSuggestData_rowcount][5] = false
				-- column 6 - tabard type
				tempSuggestData[tempSuggestData_rowcount][6] = taTabardData[maintableRow][5]
				-- column 7 - alliance or horde indicator
				tempSuggestData[tempSuggestData_rowcount][7] = taTabardData[maintableRow][6]
				-- column 8 - suggestion test string to be displayed
				tempSuggestData[tempSuggestData_rowcount][8] = taSuggestString
			end
		end
	end
	TabardAddictTab4.taTabardTotalShowing:SetText(string.format(L.TABARDS_SUGGESTED, tempSuggestData_rowcount))
end


-----------------------------------------------------------------------------
-- Sort the DisplayTable and SuggestionsTable based on tabard name.
-----------------------------------------------------------------------------
local function TabardAddict_SortDisplayTable()

	-- currently sorting only on tabard name
	table.sort(tempDisplayData, function(a,b) return a[2]<b[2] end)
	
end
--
local function TabardAddict_SortSuggestionsTable()

	-- currently sorting only on tabard name
	table.sort(tempSuggestData, function(a,b) return a[2]<b[2] end)
	
end


-----------------------------------------------------------------------------
-- Remove old unused config variables by setting them to nil.
-----------------------------------------------------------------------------
local function TabardAddict_CleanConfigVars()
	
	local key, value

	-- loop through the client stored variables - remove those no longer used
	for key,value in pairs(TabardAddictConfig_v2) do 
		if (taSettingsDefaults[key] == nil) then
			TabardAddictConfig_v2[key] = nil
			if (debugMode) then
				print("Tabard Addict - Removing old config value: " .. key)
			end
		end
	end
	
end
 
-----------------------------------------------------------------------------
-- Checks each tabard and if the game reports it has been equipped or not.
-- Data saved in global TabardAddictServerData.
-----------------------------------------------------------------------------
function TabardAddict_GetServerData()

	local r,b,visualID,sourceID,sourceInfo
	
	if (debugMode) then
		print(tostring(GetTime()) .. " === GetServerData ===")
	end

	wipe(TabardAddictServerData)
	
	-- force update of Display table
	TabardAddictUpdateDisplayTable = true
	-- force update of Suggest table
	TabardAddictUpdateSuggestTable = true
	
	-- loop through all tabards
	for b=1,TABARD_ADDICT_DATA_ROWS
	do
		sourceInfo = nil

		-- itemID reference
		r = taTabardData[b][1]

		-- Get sourceID from itemID - 1st element of table
		visualID, sourceID = C_TransmogCollection.GetItemInfo(r)

		-- save sourceID to former criteriaID slot and GetSourceInfo
		if (sourceID) then
			taTabardData[b][7] = sourceID 
			sourceInfo = C_TransmogCollection.GetSourceInfo(sourceID)
		else
			taTabardData[b][7] = 0
		end
		
		if ( sourceInfo and (sourceInfo.isCollected == true or sourceInfo.isCollected == false) ) then
			TabardAddictServerData[r] = sourceInfo.isCollected -- save "collected" true/false
			-- update tooltip table
			TabardAddictServerData_tooltiptable[r] = sourceInfo.isCollected  -- save "collected" true/false
		else
			TabardAddictServerData[r] = 'na' -- this will cause tooltip to show 'not tracked'
			-- update tooltip table
			TabardAddictServerData_tooltiptable[r] = 'na'  -- this will cause tooltip to show 'not tracked'
		end

		if (TabardAddictServerData[r] == true) then
			-- increment total tabards equipped
			tabardsUnlocked = tabardsUnlocked + 1
		end
	end
	-- Build tooltip data
	TabardAddict_BuildTooltipData()
end


-----------------------------------------------------------------------------
-- Sets the tabard text and graphics to be displayed on each row of the 
-- addon. This is for the main tabard display tab.
-----------------------------------------------------------------------------
function TabardAddictScrollBar_Update()
	local line -- 1 through 8 of our window to scroll
	local lineplusoffset -- an index into our data calculated from the scroll offset
	
	FauxScrollFrame_Update(TabardAddictScrollBar,tempDisplayData_rowcount,8,40)
	for line=1,8 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(TabardAddictScrollBar)

		-- set alpha values for each row
		if (tempDisplayData_rowcount == 0) then
			getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:Hide()
			getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:Hide()
		else
			getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:Show()
			getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:Show()
			getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:SetAlpha(0.35)
			getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:SetAlpha(0.70)
		end

		-- set display text and graphics for each row
		if lineplusoffset <= tempDisplayData_rowcount then -- check offset into valid row
			getglobal("TabardAddictEntry"..line):Enable()
			getglobal("TabardAddictEntry"..line).TabardName:SetText(tempDisplayData[lineplusoffset][2])
			getglobal("TabardAddictEntry"..line).TabardName:Show()
			
			-- check if equipped
			if (tempDisplayData[lineplusoffset][5] == true) then
				getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.BEEN_EQUIPPED.."|cff00FF00"..L.YES.."|r")
				-- show overlay on equipped tabards
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:Show()
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:SetDrawLayer("OVERLAY",5)
			else
				-- check if source id is zero - not counted toward achievement
				if (tempDisplayData[lineplusoffset][8] == 0) then
					getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.BEEN_EQUIPPED.."|cff808080"..L.NOT_TRACKED.."|r")
				else
					getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.BEEN_EQUIPPED.."|cffFF0000"..L.NO.."|r")
				end
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:Hide()
			end
			getglobal("TabardAddictEntry"..line).TabardEquipped:Show()
			
			-- set tabard icon texture
			getglobal("TabardAddictEntry"..line).TabardIcon:SetTexture(tempDisplayData[lineplusoffset][4])
			getglobal("TabardAddictEntry"..line).TabardIcon:Show()
			
			-- set local vars
			CurrentTabardEntries_itemID[line] = tempDisplayData[lineplusoffset][1]
			CurrentTabardEntries_itemLink[line] = tempDisplayData[lineplusoffset][3]
			
			-- set faction icon
			if (tempDisplayData[lineplusoffset][7] == 2) then
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png")
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Show()
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7)
			elseif (tempDisplayData[lineplusoffset][7] == 1) then
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png")
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Show()
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7)
			else
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Hide()
			end
			
		else
			getglobal("TabardAddictEntry"..line):Disable()
			getglobal("TabardAddictEntry"..line).TabardName:Hide()
			getglobal("TabardAddictEntry"..line).TabardEquipped:Hide()
			getglobal("TabardAddictEntry"..line).TabardIcon:Hide()
			getglobal("TabardAddictEntry"..line).TabardIconOverlay:Hide()
			getglobal("TabardAddictEntry"..line).TabardFactionIcon:Hide()
			-- set local vars
			CurrentTabardEntries_itemID[line] = 0
			CurrentTabardEntries_itemLink[line] = 0
			
			-- warn user
			if ((tempDisplayData_rowcount == 0) and (line == 2)) then
				getglobal("TabardAddictEntry"..line).TabardName:SetText(L.ERROR1)
				getglobal("TabardAddictEntry"..line).TabardName:Show()
				getglobal("TabardAddictEntry"..line).TabardEquipped:SetText(L.ERROR1A)
				getglobal("TabardAddictEntry"..line).TabardEquipped:Show()
			end

		end -- end if lineplusoffset <= tempDisplayData_rowcount
	end -- end for line=1,8
end


-----------------------------------------------------------------------------
-- Sets the tabard text and graphics to be displayed on each row of the 
-- addon. This is for the main tabard display tab.
-----------------------------------------------------------------------------
function TabardAddictScrollBar2_Update()
	
	local line -- 1 through 8 of our window to scroll
	local lineplusoffset -- an index into our data calculated from the scroll offset
	
	FauxScrollFrame_Update(TabardAddictScrollBar2,tempSuggestData_rowcount,8,40)

	for line=1,8 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(TabardAddictScrollBar2)
		
		-- set alpha values for each row
		getglobal("TabardAddictSugg"..line).TabardAddictButtonBack:SetAlpha(0.35)
		getglobal("TabardAddictSugg"..line).TabardAddictButtonHighlight:SetAlpha(0.70)
		
		if (lineplusoffset <= tempSuggestData_rowcount) then
			getglobal("TabardAddictSugg"..line):Enable()
			getglobal("TabardAddictSugg"..line).TabardName:SetText(tempSuggestData[lineplusoffset][2])
			getglobal("TabardAddictSugg"..line).TabardName:Show()

			-- set suggestion text
			local displayStr = tempSuggestData[lineplusoffset][8]
			getglobal("TabardAddictSugg"..line).TabardEquipped:SetText(displayStr)
			getglobal("TabardAddictSugg"..line).TabardEquipped:Show()
			
			-- set tabard icon texture
			getglobal("TabardAddictSugg"..line).TabardIcon:SetTexture(tempSuggestData[lineplusoffset][4])
			getglobal("TabardAddictSugg"..line).TabardIcon:Show()
			
			-- set overlay texture??
			--getglobal("TabardAddictSugg"..line).TabardIconSuggOverlay:Show()
			
			-- set local vars
			CurrentTabardSuggest_itemID[line] = tempSuggestData[lineplusoffset][1]
			CurrentTabardSuggest_itemLink[line] = tempSuggestData[lineplusoffset][3]
			
			-- set faction icon
			if (tempSuggestData[lineplusoffset][7] == 2) then
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png")
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:Show()
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7)
			elseif (tempSuggestData[lineplusoffset][7] == 1) then
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png")
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:Show()
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7)
			else
				getglobal("TabardAddictSugg"..line).TabardFactionIcon:Hide()
			end
			
		else
			getglobal("TabardAddictSugg"..line):Disable()
			getglobal("TabardAddictSugg"..line).TabardName:Hide()
			getglobal("TabardAddictSugg"..line).TabardEquipped:Hide()
			getglobal("TabardAddictSugg"..line).TabardIcon:Hide()
			getglobal("TabardAddictSugg"..line).TabardIconOverlay:Hide()
			getglobal("TabardAddictSugg"..line).TabardFactionIcon:Hide()
			-- set local vars
			CurrentTabardSuggest_itemID[line] = 0
			CurrentTabardSuggest_itemLink[line] = 0
			
			-- if no suggestions to show - display message to user
			if ((tempSuggestData_rowcount == 0) and (line == 2)) then
				getglobal("TabardAddictSugg"..line).TabardName:SetText(L.ERROR2)
				getglobal("TabardAddictSugg"..line).TabardName:Show()
			end
		end -- end if (lineplusoffset <= tempSuggestData_rowcount)

	end -- end for line=1,8
end


-----------------------------------------------------------------------------
-- Event handler for addon.
--
-- handled events:		PLAYER_EQUIPMENT_CHANGED
-- 						ADDON_LOADED
--						UPDATE_FACTION
-----------------------------------------------------------------------------
function TabardAddict_OnEvent(self, event, ...)
	

	-- Check if player changed tabard
	if (event == "PLAYER_EQUIPMENT_CHANGED") then

		local slot, hasItem = ...
		--if hasItem == nil then 
		--	hasItem = false
		--end
		
		-- 9.0 added this event back in but hasItem seems to be opposite value as expected

		-- addon is shown - do an update and return to tab1
		if frameTabardAddict:IsShown() then	
			-- if tabard slot and item put into slot - refresh listing and count
			if ( (slot == INVSLOT_TABARD) and (hasItem == false) ) then
				TabardAddictInfoFrame:Hide()
				TabardAddictDoServerQuery = true
				TabardAddict_OnShow(self)
			end
			if (debugMode) then
				print("Registered Event = " .. event .. "; slot = " .. slot .. " hasItem = " .. (hasItem and 'true' or 'false'))
			end
		else
			-- addon not shown - updated on next show event
			if ( (slot == INVSLOT_TABARD) and (hasItem == false) ) then
				TabardAddictInfoFrame:Hide()
				TabardAddictDoServerQuery = true
			end
			if (debugMode) then
				print("Not Shown - Registered Event = " .. event .. "; slot = " .. slot .. " hasItem = " .. (hasItem and 'true' or 'false'))
			end
		end
	end

	-- Check if tabard unlock status has changed
	if ((event == TRANSMOG_COLLECTION_SOURCE_ADDED) or (event == TRANSMOG_COLLECTION_SOURCE_REMOVED)) then
		local sourceID = ...
		
		if sourceID == nil then
			sourceID = 0
		end

		-- check if categoryID is 6 (tabard)
		if (sourceID > 0) then
			local categoryID = C_TransmogCollection.GetAppearanceSourceInfo(sourceID)
			-- update data if a tabard has been added/removed
			if ( categoryID and categoryID == Enum.TransmogCollectionType.Tabard + 1) then
				-- addon is shown
				if frameTabardAddict:IsShown() then	
					TabardAddictInfoFrame:Hide()
					TabardAddictDoServerQuery = true
					TabardAddict_OnShow(self)
					if (debugMode) then
						print("Registered Event = " .. event .. "; sourceID = " .. tostring(sourceID))
					end
				else -- addon not shown
					TabardAddictInfoFrame:Hide()
					TabardAddictDoServerQuery = true
					if (debugMode) then
						print("Not Shown - Registered Event = " .. event .. "; sourceID = " .. tostring(sourceID))
					end
				end
			end
		end	
	end -- end event TRANSMOG_COLLECTION_SOURCE

	if (event == "ADDON_LOADED") then
		local arg1 = ...
		
		if (arg1 == "TabardAddict") then
			--Show/Hide character frame launcher
			frameTabardAddictCharFrameLauncher:Show()
			
			-- check for nil global config values - if not exist then default to on (true)
			if (TabardAddictConfig_v2 == nil) then
				TabardAddictConfig_v2 = taSettingsDefaults
			end

			-- check for nil on each config item - in case a new one is added
			if (TabardAddictConfig_v2.CB_AO == nil) then
				TabardAddictConfig_v2.CB_AO = taSettingsDefaults.CB_AO
			end
			if (TabardAddictConfig_v2.CB_HO == nil) then
				TabardAddictConfig_v2.CB_HO  = taSettingsDefaults.CB_HO
			end
			if (TabardAddictConfig_v2.CB_TCG == nil) then
				TabardAddictConfig_v2.CB_TCG = taSettingsDefaults.CB_TCG
			end
			if (TabardAddictConfig_v2.CB_Other == nil) then
				TabardAddictConfig_v2.CB_Other = taSettingsDefaults.CB_Other
			end
			if (TabardAddictConfig_v2.CB_NA == nil) then
				TabardAddictConfig_v2.CB_NA = taSettingsDefaults.CB_NA
			end
			if (TabardAddictConfig_v2.CB_PVP == nil) then
				TabardAddictConfig_v2.CB_PVP = taSettingsDefaults.CB_PVP
			end
			if (TabardAddictConfig_v2.CB_BCF == nil) then
				TabardAddictConfig_v2.CB_BCF = taSettingsDefaults.CB_BCF
			end
			if (TabardAddictConfig_v2.CB_WotLKF == nil) then
				TabardAddictConfig_v2.CB_WotLKF = taSettingsDefaults.CB_WotLKF
			end
			if (TabardAddictConfig_v2.CB_CF == nil) then
				TabardAddictConfig_v2.CB_CF = taSettingsDefaults.CB_CF
			end
			if (TabardAddictConfig_v2.CB_MOPF == nil) then
				TabardAddictConfig_v2.CB_MOPF = taSettingsDefaults.CB_MOPF
			end
			if (TabardAddictConfig_v2.CB_WOD == nil) then
				TabardAddictConfig_v2.CB_WOD = taSettingsDefaults.CB_WOD
			end
			if (TabardAddictConfig_v2.CB_LG == nil) then
				TabardAddictConfig_v2.CB_LG = taSettingsDefaults.CB_LG
			end
			if (TabardAddictConfig_v2.CB_BFA == nil) then
				TabardAddictConfig_v2.CB_BFA = taSettingsDefaults.CB_BFA
			end
			if (TabardAddictConfig_v2.CB_SLANDS == nil) then
				TabardAddictConfig_v2.CB_SLANDS = taSettingsDefaults.CB_SLANDS
			end
			if (TabardAddictConfig_v2.CB_DFLIGHT == nil) then
				TabardAddictConfig_v2.CB_DFLIGHT = taSettingsDefaults.CB_DFLIGHT
			end
			if (TabardAddictConfig_v2.CB_SUGG_ACH == nil) then
				TabardAddictConfig_v2.CB_SUGG_ACH = taSettingsDefaults.CB_SUGG_ACH
			end
			if (TabardAddictConfig_v2.CB_SUGG_QST == nil) then
				TabardAddictConfig_v2.CB_SUGG_QST = taSettingsDefaults.CB_SUGG_QST
			end
			if (TabardAddictConfig_v2.CB_SUGG_CUR == nil) then
				TabardAddictConfig_v2.CB_SUGG_CUR = taSettingsDefaults.CB_SUGG_CUR
			end
			if (TabardAddictConfig_v2.CB_INFO_EQUIP == nil) then
				TabardAddictConfig_v2.CB_INFO_EQUIP = taSettingsDefaults.CB_INFO_EQUIP
			end
			if (TabardAddictConfig_v2.CB_FILTER_EQYES == nil) then
				TabardAddictConfig_v2.CB_FILTER_EQYES = taSettingsDefaults.CB_FILTER_EQYES
			end
			if (TabardAddictConfig_v2.CB_FILTER_EQNO == nil) then
				TabardAddictConfig_v2.CB_FILTER_EQNO = taSettingsDefaults.CB_FILTER_EQNO
			end
			if (TabardAddictConfig_v2.CB_FILTER_NTBG == nil) then
				TabardAddictConfig_v2.CB_FILTER_NTBG = taSettingsDefaults.CB_FILTER_NTBG
			end

			-- cleanup old config vars
			TabardAddict_CleanConfigVars()
			
		end
	end
	if (event == "UPDATE_FACTION") then
		-- do server query in case player has gained enough rep for a new tabard
		TabardAddictDoServerQuery = true
	end
end


-----------------------------------------------------------------------------
-- OnEnter handler for the tabard entry and suggestion rows.
--
-- @param line  		line of the addon that throws OnEnter event.
-----------------------------------------------------------------------------
function TabardAddictEntryButton_OnEnter(line)
	
	if (debugMode) then
		print(tostring(GetTime()) .. " EntryButton - OnEnter handled - line:" .. tostring(line))
	end

end
--
function TabardAddictSuggButton_OnEnter(line)

	if (debugMode) then
		print(tostring(GetTime()) .. " SuggButton - OnEnter handled - line:" .. tostring(line))
	end

end


-----------------------------------------------------------------------------
-- OnLeave handler for the tabard entry and suggestion rows.
-----------------------------------------------------------------------------
function TabardAddictEntryButton_OnLeave()
	ResetCursor()
	GameTooltip:Hide()
	
	if (debugMode) then
		print(tostring(GetTime()) .. " EntryButton - OnLeave handled")
	end

end
--
function TabardAddictSuggButton_OnLeave()
	ResetCursor()
	GameTooltip:Hide()
	
	if (debugMode) then
		print(tostring(GetTime()) .. " SuggButton - OnLeave handled")
	end
	
end

-----------------------------------------------------------------------------
-- OnUpdate handlers for tabard entry and suggesstion lines.
--
-- @param line  		line of the addon that throws OnEnter event.
-----------------------------------------------------------------------------
function TabardAddictEntryButton_OnUpdate(line)
	
	local itemID = CurrentTabardEntries_itemID[line]
	
	-- Show tooltip if over tabard icon
	if ( (MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon)) and (itemID > 0) ) then
		-- Hide info frame
		TabardAddictInfoFrame:Hide()
		
		GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120)
		GameTooltip:SetHyperlink("item:"..itemID)
		GameTooltip:Show()
		-- if ctrl is down show inspect cursor else reset it
		if ( IsModifiedClick("DRESSUP") ) then
			ShowInspectCursor()
		else
			ResetCursor()
		end
		-- reset main display text
		TabardAddictTab1.taDescText:SetText(string.format(L.TABARD_TITLE, UnitName("player")))
	else
		-- not over icon - hide tooltip and reset cursor
		GameTooltip:Hide()
		ResetCursor()

		-- if info window is not shown - show help text
		if (TabardAddictInfoFrame:IsShown() ~= true) then
			GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120)
			GameTooltip:SetText(L.DETAILWINDOW_TT, 1.0, 1.0, 1.0, 0.85, 1)
		end
	end
	
end
--
function TabardAddictSuggButton_OnUpdate(line)

	local itemID = CurrentTabardSuggest_itemID[line]
	
	-- Show tooltip if over tabard icon
	if ( (MouseIsOver(getglobal("TabardAddictSugg"..line).TabardIcon)) and (itemID > 0) ) then
		-- Hide info frame
		TabardAddictInfoFrame:Hide()
		
		GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120)
		GameTooltip:SetHyperlink("item:"..itemID)
		GameTooltip:Show()
		-- if ctrl is down show inspect cursor else reset it
		if ( IsModifiedClick("DRESSUP") ) then
			ShowInspectCursor()
		else
			ResetCursor()
		end
		-- reset main display text
		TabardAddictTab4.taSuggestDescText:SetText(L.SUGGESTIONS_TITLE)
	else
		-- not over icon - hide tooltip and reset cursor
		GameTooltip:Hide()
		ResetCursor()

		-- if info window is not shown - show help text
		if (TabardAddictInfoFrame:IsShown() ~= true) then
			GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120)
			GameTooltip:SetText(L.DETAILWINDOW_TT, 1.0, 1.0, 1.0, 0.85, 1)
		end

	end
end


-----------------------------------------------------------------------------
-- ModifiedClick handler. Calls functions that launch dressup window or 
-- inserts chat link for item.
--
-- @param line  		Entry or Suggest line that received click.
-----------------------------------------------------------------------------
function TabardAddictEntryButton_OnModifiedClick(line)
	local itemID = CurrentTabardEntries_itemID[line]
	local itemLink = CurrentTabardEntries_itemLink[line]
	
	-- If there was a DRESSUP or CHATLINK click over a tabard icon handle it
	if ( (MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon)) and (itemID > 0) ) then
		if ( IsModifiedClick("DRESSUP") ) then
			DressUpItemLink(itemLink)
		elseif ( IsModifiedClick("CHATLINK") ) then
			if (ChatEdit_GetActiveWindow()) then
				print (itemLink)
				ChatEdit_InsertLink(itemLink)
			end
		end
	end
end
--
function TabardAddictSuggButton_OnModifiedClick(line)
	local itemID = CurrentTabardSuggest_itemID[line]
	local itemLink = CurrentTabardSuggest_itemLink[line]
	
	-- If there was a DRESSUP or CHATLINK click over a tabard icon handle it
	if ( (MouseIsOver(getglobal("TabardAddictSugg"..line).TabardIcon)) and (itemID > 0) ) then
		if ( IsModifiedClick("DRESSUP") ) then
			DressUpItemLink(itemLink)
		elseif ( IsModifiedClick("CHATLINK") ) then
			if (ChatEdit_GetActiveWindow()) then
				ChatEdit_InsertLink(itemLink)
			end
		end
	end
end


-----------------------------------------------------------------------------
-- OnClick handler for the tabs along bottom of addon. Switches view to 
-- clicked tab.
--
-- @param tabID  		ID of clicked tab.
-----------------------------------------------------------------------------
function TabardAddictTab_OnClick(self, tabID)
	if ( not tabID ) then
		tabID = self:GetID()
	end

	-- clear search
	if (taNameFilter ~= "") then
		taNameFilter = ""
		TabardAddictTab1.SearchBox:SetText("")
		TabardAddictUpdateDisplayTable = true
	end
	-- set search widget back to default display
	TabardAddictTab1.SearchBox:Hide()
	TabardAddictTab1.SearchBox.clearButton:Hide()
	TabardAddictTab1.taDescText:Show()

	-- Hide info frame
	if (TabardAddictInfoFrame:IsShown()) then
		TabardAddictInfoFrame:Hide()
	end
	
	if ( tabID == 1 ) then
		-- Tabards tab clicked
		TabardAddictScrollBar:Show()
		TabardAddictScrollBar2:Hide()
		TabardAddictTab1:Show()
		TabardAddictTab2:Hide()
		TabardAddictTab3:Hide()
		TabardAddictTab4:Hide()
		-- populate the display table
		TabardAddict_PopulateDisplayTable()
		-- sort the display table
		TabardAddict_SortDisplayTable()
		-- initial display
		TabardAddictScrollBar_Update()
		TabardAddictShowTabards()
		TabardAddictHideSuggestions()
		PanelTemplates_SetTab(frameTabardAddict, 1)
	elseif ( tabID == 2 ) then
		-- Config tab clicked
		TabardAddictScrollBar:Hide()
		TabardAddictScrollBar2:Hide()
		TabardAddictTab1:Hide()
		TabardAddictTab2:Show()
		TabardAddictTab3:Hide()
		TabardAddictTab4:Hide()
		-- set checkbox values
		CheckButtonAO:SetChecked(TabardAddictConfig_v2.CB_AO)
		CheckButtonHO:SetChecked(TabardAddictConfig_v2.CB_HO)
		CheckButtonSuggAch:SetChecked(TabardAddictConfig_v2.CB_SUGG_ACH)
		CheckButtonSuggQst:SetChecked(TabardAddictConfig_v2.CB_SUGG_QST)
		CheckButtonSuggCur:SetChecked(TabardAddictConfig_v2.CB_SUGG_CUR)
		CheckButtonPVP:SetChecked(TabardAddictConfig_v2.CB_PVP)
		CheckButtonTCG:SetChecked(TabardAddictConfig_v2.CB_TCG)
		CheckButtonNA:SetChecked(TabardAddictConfig_v2.CB_NA)
		CheckButtonOther:SetChecked(TabardAddictConfig_v2.CB_Other)

		TabardAddictHideTabards()
		TabardAddictHideSuggestions()
		PanelTemplates_SetTab(frameTabardAddict, 3) -- 3rd tab
	elseif ( tabID == 3 ) then
		-- About tab clicked
		TabardAddictScrollBar:Hide()
		TabardAddictScrollBar2:Hide()
		TabardAddictTab1:Hide()
		TabardAddictTab2:Hide()
		TabardAddictTab3:Show()
		TabardAddictTab4:Hide()
		TabardAddictHideTabards()
		TabardAddictHideSuggestions()
		PanelTemplates_SetTab(frameTabardAddict, 4) -- 4th tab
	elseif ( tabID == 4 ) then	
		-- Suggestions tab clicked
		TabardAddictScrollBar:Hide()
		TabardAddictScrollBar2:Show()
		TabardAddictTab1:Hide()
		TabardAddictTab2:Hide()
		TabardAddictTab3:Hide()
		TabardAddictTab4:Show()
		-- populate the suggestions table
		TabardAddict_PopulateSuggestionsTable()
		-- sort the suggestions table
		TabardAddict_SortSuggestionsTable()
		-- initial display
		TabardAddictHideTabards()
		TabardAddictShowSuggestions()
		TabardAddictScrollBar2_Update()
		PanelTemplates_SetTab(frameTabardAddict, 2) -- 2nd tab
	end
	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB)
end


-----------------------------------------------------------------------------
-- Handles OnClick for ENTRY or SUGGESTION rows of addon.
--
-- @param line  		ENTRY or SUGGESTION row that was clicked.
-- @param section  		0 is ENTRY section and 1 is SUGGESTION section.
-----------------------------------------------------------------------------
function TabardAddictEntryButton_OnClick(self, line, section)

	-- Only show info window if click was not over tabard icon
	if ( MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon) ~= true ) then

		PlaySound(SOUNDKIT.IG_INVENTORY_ROTATE_CHARACTER)

		if (TabardAddictInfoFrame:IsShown() ~= true) then
			TabardAddictInfoFrame:Show()
			CheckButtonInfoDressUp.fs:SetText(L.CONFIG_INFO_EQUIP)
			CheckButtonInfoDressUp:SetChecked(TabardAddictConfig_v2.CB_INFO_EQUIP)
		end

		-- Get Item ID
		local itemID = 0, itemLink, itemName 
		InfoFrameCurrent_itemID = 0
		InfoFrameCurrent_itemLink = ""

		if (section == 0) then
			if (line > 0) then
				itemID = CurrentTabardEntries_itemID[line]
				itemLink = CurrentTabardEntries_itemLink[line]
				itemName = getglobal("TabardAddictEntry"..line).TabardName:GetText()
			else 
				return
			end
		elseif (section == 1) then
			if (line > 0) then
				itemID = CurrentTabardSuggest_itemID[line]
				itemLink = CurrentTabardSuggest_itemLink[line]
				itemName = getglobal("TabardAddictSugg"..line).TabardName:GetText()
			else 
				return
			end
		else -- only section 0 (ENTRY) and 1 (SUGGESTION) currently
			return
		end

		TabardAddictInfoFrame.TabardName:SetText(itemName)
		local _, _, fontFlags = TabardAddictInfoFrame.EditBox:GetFont();
		TabardAddictInfoFrame.EditBox:SetFont("Fonts\\ARIALN.TTF", 13, fontFlags)
		TabardAddictInfoFrame.EditBox:SetText(L.ITEM_INFO_URL..tostring(itemID))
		-- Text can potentially wrap out of view so setcursor position to 0
		TabardAddictInfoFrame.EditBox:SetCursorPosition(0)

		-- save Item ID
		InfoFrameCurrent_itemID = itemID
		InfoFrameCurrent_itemLink = itemLink
		
		-- TryOn item
		if (CheckButtonInfoDressUp:GetChecked() == true) then
			TabardAddictDressUpModel:Dress()
		else
			TabardAddictDressUpModel:Undress()
		end
		-- In WoW 7.0+ (Legion) - TryOn appears to no longer work with itemID but works with itemLink
		-- verify again in future patch
		-- TabardAddictDressUpModel:TryOn(itemID)
		TabardAddictDressUpModel:TryOn(itemLink)
		
		-- debug
		if (debugMode) then
			local x,y,z = TabardAddictDressUpModel:GetPosition()
			print("InfoFrame:IsShown(): " .. tostring(TabardAddictInfoFrame:IsShown()))
			print("Model x,y,z position = " .. tostring(x) .. ", " .. tostring(y) .. ", " .. tostring(z))
			print("itemID: " .. tostring(itemID))
			print("itemLink: " .. itemLink)
		end

	end -- end if ( MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon) ~= true )

end


-----------------------------------------------------------------------------
-- Handle OnClick of ShowEquipment button on DressUp window. Will Undress or
-- Dress the character and then tryon the tabard.
-----------------------------------------------------------------------------
function TabardAddict_InfoFrameShowEquipment_OnClick(self)
	
	if self.checked then
        PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
    else 
        PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
    end

	-- Get Tabard Item
	if (CheckButtonInfoDressUp:GetChecked() == true) then
		TabardAddictConfig_v2.CB_INFO_EQUIP = true -- save config setting 
		TabardAddictDressUpModel:Dress()
		--TabardAddictDressUpModel:TryOn(InfoFrameCurrent_itemID)
		TabardAddictDressUpModel:TryOn(InfoFrameCurrent_itemLink)
	else
		TabardAddictConfig_v2.CB_INFO_EQUIP = false -- save config setting
		TabardAddictDressUpModel:Undress()
		--TabardAddictDressUpModel:TryOn(InfoFrameCurrent_itemID)
		TabardAddictDressUpModel:TryOn(InfoFrameCurrent_itemLink)
	end

end


-----------------------------------------------------------------------------
-- Handles the OnClick of the launcher button in the character window next
-- to the tabard item.
-----------------------------------------------------------------------------
function TabardAddictLauncher_OnClick(self)

	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
	
	if frameTabardAddict:IsShown() then
		frameTabardAddict:Hide()
	else
		frameTabardAddict:Show()
	end
end


-----------------------------------------------------------------------------
-- OnClick handler for the search button. Displays or hides the search box.
-----------------------------------------------------------------------------
function TabardAddictSearchToggle_OnClick(self)

	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB)
	
	-- show or hide - edit box / header text
	if TabardAddictTab1.SearchBox:IsShown() then
		-- clear search
		if (taNameFilter ~= "") then
			taNameFilter = ""
			TabardAddictTab1.SearchBox:SetText("")
			-- populate the display table - force repopulation
			TabardAddictUpdateDisplayTable = true
			TabardAddict_PopulateDisplayTable()
			-- sort the display table
			TabardAddict_SortDisplayTable()
			-- initial display
			TabardAddictScrollBar_Update()
		end
		TabardAddictTab1.SearchBox:Hide()
		TabardAddictTab1.SearchBox.clearButton:Hide()
		TabardAddictTab1.taDescText:Show()
	else
		TabardAddictTab1.taDescText:Hide()
		TabardAddictTab1.SearchBox:Show()
		TabardAddictTab1.SearchBox:SetFocus()
	end

end


-----------------------------------------------------------------------------
-- OnClick handler for clear search button.
-----------------------------------------------------------------------------
function TabardAddictSearchClear_OnClick(self)

	-- clear search
	if (taNameFilter ~= "") then
		taNameFilter = ""
		TabardAddictTab1.SearchBox:SetText("")
		-- populate the display table - force repopulation
		TabardAddictUpdateDisplayTable = true
		TabardAddict_PopulateDisplayTable()
		-- sort the display table
		TabardAddict_SortDisplayTable()
		-- initial display
		TabardAddictScrollBar_Update()
	end

	self:ClearFocus()
    self:SetFontObject("GameFontDisable")
    self.searchIcon:SetVertexColor(0.6, 0.6, 0.6)
    self.clearButton:Hide()

end


-----------------------------------------------------------------------------
-- Main OnShow event handler for the addon.
-----------------------------------------------------------------------------
function TabardAddict_OnShow(self)
	
	local tabardsNextAchievement = 0
	
	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_OPEN)
	
	-- set description text
	TabardAddictTab1.taDescText:SetText(string.format(L.TABARD_TITLE, UnitName("player")))
	-- set search widget back to default display
	TabardAddictTab1.SearchBox:Hide()
	TabardAddictTab1.SearchBox.clearButton:Hide()
	TabardAddictTab1.taDescText:Show()

	if (TabardAddictDoServerQuery == true) then
		-- reset # of equipped tabards
		tabardsUnlocked = 0
		-- query server for data
		TabardAddict_GetServerData()
		TabardAddictDoServerQuery = false
	end

	-- populate counts
	if (tabardsUnlocked >= 30) then
		tabardsNextAchievement = "NA"
	elseif (tabardsUnlocked >= 25) then
		tabardsNextAchievement = 30
	elseif (tabardsUnlocked >= 10) then
		tabardsNextAchievement = 25
	elseif (tabardsUnlocked >= 1) then
		tabardsNextAchievement = 10
	else
		tabardsNextAchievement = 1
	end
	
	-- populate the display table
	TabardAddict_PopulateDisplayTable()
	-- sort the display table
	TabardAddict_SortDisplayTable()
	-- initial display
	TabardAddictScrollBar_Update()
	-- populate the suggestions table
	TabardAddict_PopulateSuggestionsTable()
	-- sort the suggestions table
	TabardAddict_SortSuggestionsTable()
	
	
	
		-- populate equipped count and next achievement text
	if (tabardsUnlocked >= 30) then
		TabardAddictTab1.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS_2, tabardsUnlocked))
		TabardAddictTab4.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS_2, tabardsUnlocked))
	else
		TabardAddictTab1.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS, tabardsUnlocked, tabardsNextAchievement))
		TabardAddictTab4.taTabardTotalText:SetText(string.format(L.TABARD_COUNTS, tabardsUnlocked, tabardsNextAchievement))
	end
	
	-- show/hide panels
	TabardAddictTab1:Show()
	TabardAddictTab2:Hide()
	TabardAddictTab3:Hide()
	TabardAddictTab4:Hide()
	PanelTemplates_SetTab(frameTabardAddict, 1)
	TabardAddictShowTabards()
	TabardAddictHideSuggestions()
	
	--set DressUp model and position
	TabardAddictDressUpModel:SetUnit("player")
	local settings
	local hasAlternateForm, inAlternateForm = C_PlayerInfo.GetAlternateFormInfo()
	if ( (hasAlternateForm==true) and (inAlternateForm==true) ) then
		settings = taDressUpSettings[playerRaceSex.."Alt"]
		-- debug
		if (debugMode) then
			print("loading: " .. playerRaceSex .. "Alt")
		end
	else
		settings = taDressUpSettings[playerRaceSex]
		-- debug
		if (debugMode) then
			print("loading: " .. playerRaceSex)
		end
	end
	
	TabardAddictDressUpModel:SetPortraitZoom(settings.zoom)
	TabardAddictDressUpModel.zoomLevel  = settings.zoom
	TabardAddictDressUpModel:SetPosition(settings.x, settings.y, settings.z)
	
end


-----------------------------------------------------------------------------
-- Show the launcher button after OnLoad event.
-----------------------------------------------------------------------------
function TabardAddictLauncher_OnLoad(self)

	--Show/Hide character frame launcher
	frameTabardAddictCharFrameLauncher:Show()

end


-----------------------------------------------------------------------------
-- Handle the OnHide event for the addon.
-----------------------------------------------------------------------------
function TabardAddict_OnHide(self)

	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_CLOSE)
	-- debug
	if (debugMode) then
		print("TabardAddict_OnHide() called")
	end

	-- clear search params
	if (taNameFilter ~= "") then
		taNameFilter = ""
		TabardAddictTab1.SearchBox:SetText("")
		-- populate the display table - force repopulation
		TabardAddictUpdateDisplayTable = true
	end

	-- Hide info frame
	if (TabardAddictInfoFrame:IsShown()) then
		TabardAddictInfoFrame:Hide()
	end
	
end


-----------------------------------------------------------------------------
-- Shows each of the ENTRY and SUGGESTIONS rows.
-----------------------------------------------------------------------------
function TabardAddictShowTabards()
	local line
	for line=1,8 do
		getglobal("TabardAddictEntry"..line):Show()
	end
end
--
function TabardAddictShowSuggestions()
	local line
	for line=1,8 do
		getglobal("TabardAddictSugg"..line):Show()
	end
end


-----------------------------------------------------------------------------
-- Hides each of the ENTRY and SUGGESTIONS rows.
-----------------------------------------------------------------------------
function TabardAddictHideTabards()
	local line
	for line=1,8 do
		getglobal("TabardAddictEntry"..line):Hide()
	end
end
--
function TabardAddictHideSuggestions()
	local line
	for line=1,8 do
		getglobal("TabardAddictSugg"..line):Hide()
	end
end


-----------------------------------------------------------------------------
-- Handles the clicks of the config check boxes.
-----------------------------------------------------------------------------
function TabardAddict_SetConfigVariables(self)

	local name = self:GetName()
	
	if (debugMode) then
		print("TabardAddict_SetConfigVariables = " .. name)
	end

	-- play checkbox sound
	if self.checked then
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
	else 
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
	end

	-- update the approprate temp table based on which config clicked
    if (name == 'CheckButtonAO') then
    	TabardAddictUpdateDisplayTable = true
	elseif (name == 'CheckButtonHO') then
    	TabardAddictUpdateDisplayTable = true
	elseif (name == 'CheckButtonSuggAch') then
    	TabardAddictUpdateSuggestTable = true
    elseif (name == 'CheckButtonSuggQst') then
    	TabardAddictUpdateSuggestTable = true
    elseif (name == 'CheckButtonSuggCur') then
    	TabardAddictUpdateSuggestTable = true
	elseif (name == 'CheckButtonPVP') then
    	TabardAddictUpdateDisplayTable = true
	elseif (name == 'CheckButtonTCG') then
    	TabardAddictUpdateDisplayTable = true
	elseif (name == 'CheckButtonNA') then
    	TabardAddictUpdateDisplayTable = true
	elseif (name == 'CheckButtonOther') then
    	TabardAddictUpdateDisplayTable = true
	end

	if (CheckButtonAO:GetChecked() == true) then
		TabardAddictConfig_v2.CB_AO = true
	else
		TabardAddictConfig_v2.CB_AO = false
	end
	if (CheckButtonHO:GetChecked() == true) then
		TabardAddictConfig_v2.CB_HO = true
	else
		TabardAddictConfig_v2.CB_HO = false
	end
	if (CheckButtonSuggAch:GetChecked() == true) then
		TabardAddictConfig_v2.CB_SUGG_ACH = true
	else
		TabardAddictConfig_v2.CB_SUGG_ACH = false
	end
	if (CheckButtonSuggQst:GetChecked() == true) then
		TabardAddictConfig_v2.CB_SUGG_QST = true
	else
		TabardAddictConfig_v2.CB_SUGG_QST = false
	end
	if (CheckButtonSuggCur:GetChecked() == true) then
		TabardAddictConfig_v2.CB_SUGG_CUR = true
	else
		TabardAddictConfig_v2.CB_SUGG_CUR = false
	end
	if (CheckButtonPVP:GetChecked() == true) then
		TabardAddictConfig_v2.CB_PVP = true
	else
		TabardAddictConfig_v2.CB_PVP = false
	end
	if (CheckButtonTCG:GetChecked() == true) then
		TabardAddictConfig_v2.CB_TCG = true
	else
		TabardAddictConfig_v2.CB_TCG = false
	end
	if (CheckButtonNA:GetChecked() == true) then
		TabardAddictConfig_v2.CB_NA = true
	else
		TabardAddictConfig_v2.CB_NA = false
	end
	if (CheckButtonOther:GetChecked() == true) then
		TabardAddictConfig_v2.CB_Other = true
	else
		TabardAddictConfig_v2.CB_Other = false
	end
end

-----------------------------------------------------------------------------
-- Handle OnEnter of a config check box and display a tool tip.
-----------------------------------------------------------------------------
function TabardAddict_ConfigOnEnter(self)

	local name = self:GetName()
	
	if (debugMode) then
		print("TabardAddict_ConfigOnEnter = " .. name)
	end
	
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	-- Show tooltip based on config item name
	GameTooltip:SetText(taSettingsTooltips[name], 1.0,1.0,1.0,1,1)
end


-----------------------------------------------------------------------------
-- Handle OnEnter of the text string totals count at bottom of addon. Display
-- tooltip with more achievement details.
-----------------------------------------------------------------------------
function TabardAddict_TotalText_OnEnter(self)

	local tableRow
	
	-- Populate Achievement Completion tooltip
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 15, -50)
	GameTooltip:SetText(YELLOW_FONT_COLOR_CODE..L.ACHV_TT_TITLE..FONT_COLOR_CODE_CLOSE)
	GameTooltip:AddLine(" ")
	
	for tableRow in ipairs(tempAchievementTooltipData) do

		-- there is a case where not all data is present when tooltip data was built
		-- clients that load quickly - low level chars with not many addons
		if ((tempAchievementTooltipData[tableRow][1] == true) and 
			(tempAchievementTooltipData[tableRow][3] == nil or tempAchievementTooltipData[tableRow][3] == '')) then
			TabardAddict_BuildTooltipData()
		end

		-- build string - also show which character earned achievement
		local displayString = tempAchievementTooltipData[tableRow][2]
		if (tempAchievementTooltipData[tableRow][3] ~= nil and tempAchievementTooltipData[tableRow][3] ~= '') then
			displayString = displayString .. " (" .. tempAchievementTooltipData[tableRow][3] .. ")"
		end
		-- also show date completed
		if (tempAchievementTooltipData[tableRow][4] ~= nil) then
			displayString = displayString .. " [" .. tempAchievementTooltipData[tableRow][4] .. "]"
		end

		-- Check if achievement completed
		if (tempAchievementTooltipData[tableRow][1] == true) then
			GameTooltip:AddLine(displayString, GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b, false)
			GameTooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready")
		else
			GameTooltip:AddLine(displayString, GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, false)
			GameTooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady")
		end
	end
	
	GameTooltip:Show()
end


-----------------------------------------------------------------------------
-- Handles the OnTextChanged event of the search box.
--
-- @param userInput  	current text in the serach box.
-----------------------------------------------------------------------------
function TabardAddict_OnTextChanged(self, userInput)
	local text = self:GetText()

	if (debugMode) then
		print("TabardAddict_OnTextChanged: " .. text .. ", userInput: " .. tostring(userInput))
	end

	if (text ~= "") then
		self.label:Hide()
	else
		self.label:Show()
	end

	taNameFilter = text
	-- get rid of right and left brackets
	taNameFilter=string.gsub(taNameFilter, "%[", "")
	taNameFilter=string.gsub(taNameFilter, "%]", "")
	
	if (userInput == true) then
		-- force repopulation of display table
		TabardAddictUpdateDisplayTable = true
		-- populate the display table
		TabardAddict_PopulateDisplayTable()
		-- sort the display table
		TabardAddict_SortDisplayTable()
		-- initial display
		TabardAddictScrollBar_Update()
	end
end


-----------------------------------------------------------------------------
-- Get item info from tooltip.
--
-- @param tooltip  		reference to a tooltip object.
-----------------------------------------------------------------------------
local function OnTooltipSetItem(tooltip, data)

	-- local itemName, itemLink = tooltip:GetItem() -- Doesn't work from 10.0.2 forward on comparison tooltip - now using new tooltiputil
	local itemName, itemLink, tipId = TooltipUtil.GetDisplayedItem(tooltip)
	-- exit if itemLink is nil - client has not cached item yet
	if (itemLink == nil) then
		return
	end

	-- parse itemID from link
	local _, _, _, _, itemID, _, _, _, _, _, _, _, _, _ = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
	
	-- check if need to get data from server
	if (TabardAddictDoServerQuery == true) then
		-- reset # of equipped tabards
		tabardsUnlocked = 0
		-- query server for data
		TabardAddict_GetServerData()
		TabardAddictDoServerQuery = false
	end
	
	-- check if itemID in tooltiptable before adding text to tooltip
	if ( TabardAddictServerData_tooltiptable[tonumber(itemID)] ~= nill ) then
		if not taLineAdded then
			local ttString = L.STR_TABARD .. L.BEEN_EQUIPPED
			if (TabardAddictServerData_tooltiptable[tonumber(itemID)] == true) then
				ttString = ttString .. "|cff00FF00" .. L.YES .. "|r"
			elseif (TabardAddictServerData_tooltiptable[tonumber(itemID)] == false) then
				ttString = ttString .. "|cffFF0000" ..L.NO .. "|r"
			else
				ttString = ttString .. "|cff808080" ..L.NOT_TRACKED .. "|r"
			end
			tooltip:AddLine(ttString)
			taLineAdded = true
		end
	end
end


-----------------------------------------------------------------------------
-- Resets global taLineAdded to false.
--
-- @param tooltip  		reference to a tooltip object.
-----------------------------------------------------------------------------
local function OnTooltipCleared(tooltip, ...)
	taLineAdded = false
end


-----------------------------------------------------------------------------
-- Maps a clicked dropdown config element to internal CB value.
--
-- @param param  		parameter the received checked event.
-- @return 				returns internal CB name.
-----------------------------------------------------------------------------
local function ta_GetDDConfigValue(param)
	
	if (TabardAddictConfig_v2 == nil) then
		return
	end
	
	if (param == "CB_FILTER_EQYES") then
		return(TabardAddictConfig_v2.CB_FILTER_EQYES)
	elseif (param == "CB_FILTER_EQNO") then
		return(TabardAddictConfig_v2.CB_FILTER_EQNO)
	elseif (param == "CB_FILTER_NTBG") then
		return(TabardAddictConfig_v2.CB_FILTER_NTBG)
	elseif (param == "CB_BCF") then
		return(TabardAddictConfig_v2.CB_BCF)
	elseif (param == "CB_WotLKF") then
		return(TabardAddictConfig_v2.CB_WotLKF)
	elseif (param == "CB_CF") then
		return(TabardAddictConfig_v2.CB_CF)
	elseif (param == "CB_MOPF") then
		return(TabardAddictConfig_v2.CB_MOPF)
	elseif (param == "CB_WOD") then
		return(TabardAddictConfig_v2.CB_WOD)
	elseif (param == "CB_LG") then
		return(TabardAddictConfig_v2.CB_LG)
	elseif (param == "CB_BFA") then
		return(TabardAddictConfig_v2.CB_BFA)
	elseif (param == "CB_SLANDS") then
		return(TabardAddictConfig_v2.CB_SLANDS)
	elseif (param == "CB_DFLIGHT") then
		return(TabardAddictConfig_v2.CB_DFLIGHT)
	else
		return(false)
	end
end


-----------------------------------------------------------------------------
-- Sets the dropdown config values.
--
-- @param arg1  		contains the CB value.
-- @param arg2  		not used.
-- @param checked  		checked status of check box.
-- @return 				false if arg1 not found.
-----------------------------------------------------------------------------
local function ta_SetDDConfigValue(self, arg1, arg2, checked)
	PlaySound(SOUNDKIT.IG_INVENTORY_ROTATE_CHARACTER)
	
	if (arg1 == "CB_FILTER_EQYES") then
		TabardAddictConfig_v2.CB_FILTER_EQYES = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_FILTER_EQNO") then
		TabardAddictConfig_v2.CB_FILTER_EQNO = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_FILTER_NTBG") then
		TabardAddictConfig_v2.CB_FILTER_NTBG = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_BCF") then
		TabardAddictConfig_v2.CB_BCF = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_WotLKF") then
		TabardAddictConfig_v2.CB_WotLKF = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_CF") then
		TabardAddictConfig_v2.CB_CF = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_MOPF") then
		TabardAddictConfig_v2.CB_MOPF = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_WOD") then
		TabardAddictConfig_v2.CB_WOD = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_LG") then
		TabardAddictConfig_v2.CB_LG = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_BFA") then
		TabardAddictConfig_v2.CB_BFA = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_SLANDS") then
		TabardAddictConfig_v2.CB_SLANDS = checked
		TabardAddictUpdateDisplayTable = true
	elseif (arg1 == "CB_DFLIGHT") then
		TabardAddictConfig_v2.CB_DFLIGHT = checked
		TabardAddictUpdateDisplayTable = true
	else
		return(false)
	end
	
	-- after updating config refresh display
	-- populate the display table
	TabardAddict_PopulateDisplayTable()
	-- sort the display table
	TabardAddict_SortDisplayTable()
	-- display the data
	TabardAddictScrollBar_Update()
	
end


-----------------------------------------------------------------------------
-- Check All Dropdown options to true or false
--
-- @param arg1  		true or false
-- @param arg2          not used.
-- @return 				none
-----------------------------------------------------------------------------
local function ta_CheckAllExpansionsDDConfigValue(self, arg1, arg2)

	-- Set each expansion config settings to true or false
	TabardAddictConfig_v2.CB_BCF = arg1
	TabardAddictConfig_v2.CB_WotLKF = arg1
	TabardAddictConfig_v2.CB_CF = arg1
	TabardAddictConfig_v2.CB_MOPF = arg1
	TabardAddictConfig_v2.CB_WOD = arg1
	TabardAddictConfig_v2.CB_LG = arg1
	TabardAddictConfig_v2.CB_BFA = arg1
	TabardAddictConfig_v2.CB_SLANDS = arg1
	TabardAddictConfig_v2.CB_DFLIGHT = arg1

	TabardAddictUpdateDisplayTable = true
	-- after updating config refresh display
	-- populate the display table
	TabardAddict_PopulateDisplayTable()
	-- order the display
	TabardAddict_SortDisplayTable()

	-- Refresh Dropdown values after updating flags
	UIDropDownMenu_Refresh(TabardAddict_ConfigExpansions, nil, 1)

	if (debugMode) then
		print("ta_CheckAllExpansionsDDConfigValue() - arg1: " .. tostring(arg1))
	end

end


-----------------------------------------------------------------------------
-- Defines settings for FilterMenu (dropdown config) when OnLoad event from 
-- xml is handled.
-----------------------------------------------------------------------------
function TabardAddict_FilterMenu_OnLoad()
	local info =  {}
	local x
    info.isTitle = 1
	info.text = "View Filter"
	info.notCheckable = 1
	UIDropDownMenu_AddButton(info, 1)
	
	for x = 1,3 do
		info.text = FilterMenuInfo[x].text
		info.disabled = nil
		info.isTitle = nil
		info.func = ta_SetDDConfigValue
		info.arg1 = FilterMenuInfo[x].arg1
		info.notCheckable = nil
		info.checked = ta_GetDDConfigValue(FilterMenuInfo[x].arg1)
		info.keepShownOnClick = 1
		info.isNotRadio = 1
		UIDropDownMenu_AddButton(info, 1)
	end
    -- Add the above information to the options menu as a button.
	if (debugMode) then
		print("TabardAddict_FilterMenu_OnLoad")
	end
end


-----------------------------------------------------------------------------
-- Handles the OnClick for the filter drop down config.
-----------------------------------------------------------------------------
function TabardAddict_FilterMenuButton_OnClick(self)
	PlaySound(SOUNDKIT.IG_INVENTORY_ROTATE_CHARACTER)
	ToggleDropDownMenu(1, nil, TabardAddict_FilterMenu, self, 0, 0)
	if (debugMode) then
		print("TabardAddict_FilterMenuButton_OnClick")
	end
end


-----------------------------------------------------------------------------
-- Handles OnClick event of Config Expansions dropdown menu.
-----------------------------------------------------------------------------
function TabardAddict_ConfigExpansionsButton_OnClick(self)
	PlaySound(SOUNDKIT.IG_INVENTORY_ROTATE_CHARACTER)
	ToggleDropDownMenu(1, nil, TabardAddict_ConfigExpansions, self, 5, 0)
	if (debugMode) then
		print("TabardAddict_ConfigExpansionsButton_OnClick")
	end
end


-----------------------------------------------------------------------------
-- Handles OnLoad event of the configmenu dropdown.
-----------------------------------------------------------------------------
function TabardAddict_ConfigExpansions_OnLoad()

	-- Build "buttons" for drop down menu
	local info = {}
	local x
	info.isTitle = 1
	info.text = L.CONFIG_EXPANSIONS
	info.notCheckable = 1
	UIDropDownMenu_AddButton(info, 1)

	-- Check All button
	info.isTitle = nil
	info.text = CHECK_ALL
	info.disabled = nil
	info.notCheckable = 1
	info.keepShownOnClick = 1
	info.func = ta_CheckAllExpansionsDDConfigValue
	info.arg1 = true
	UIDropDownMenu_AddButton(info, 1)

	-- Uncheck All Button
	info.isTitle = nil
	info.text = UNCHECK_ALL
	info.disabled = nil
	info.notCheckable = 1
	info.keepShownOnClick = 1
	info.func = ta_CheckAllExpansionsDDConfigValue
	info.arg1 = false
	UIDropDownMenu_AddButton(info, 1)

	-- Populate expansion config dropdown
	for x = 1,#taConfigExpansionsInfo do
		info.text = taConfigExpansionsInfo[x].text
		info.disabled = nil
		info.isTitle = nil
		info.func = ta_SetDDConfigValue 
		info.arg1 = taConfigExpansionsInfo[x].arg1
		info.notCheckable = nil
		info.checked = function() 
			return ta_GetDDConfigValue(taConfigExpansionsInfo[x].arg1)
		end
		info.keepShownOnClick = 1
		info.isNotRadio = 1
		UIDropDownMenu_AddButton(info, 1)
	end

	-- Add the above information to the options menu as a button.
	if (debugMode) then
		print("TabardAddict_ConfigExpansions_OnLoad")
	end
end


-----------------------------------------------------------------------------
-- Main OnLoad function for the addon.
-----------------------------------------------------------------------------
function TabardAddict_OnLoad(self)

	if (debugMode) then
		print("TabardAddict_OnLoad: "..self:GetName())
	end

	-- Register the frame so it can be dragged
	self:RegisterForDrag("LeftButton")

	-- Register Events
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED") -- BOOL in event seems opposite true/false
	self:RegisterEvent("TRANSMOG_COLLECTION_SOURCE_ADDED")
	self:RegisterEvent("TRANSMOG_COLLECTION_SOURCE_REMOVED")
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("UPDATE_FACTION")
	
	-- HookScripts for in-game tooltips
	-- TooltipDataProcessor.AddTooltipPostCall(Enum.TooltipDataType.Item, OnTooltipSetItem)
	TooltipDataProcessor.AddTooltipPostCall(TooltipDataProcessor.AllTypes, OnTooltipSetItem)
	GameTooltip:HookScript("OnTooltipCleared", OnTooltipCleared)

	-- init variables
	TabardAddictDoServerQuery = true
	TabardAddictUpdateDisplayTable = true
	TabardAddictUpdateSuggestTable = true
	
	-- Tab Handling code
	PanelTemplates_SetNumTabs(self, 4)
	PanelTemplates_SetTab(self, 1)

	-- show/hide panels
	TabardAddictTab1:Show()
	TabardAddictTab2:Hide()
	TabardAddictTab3:Hide()
	TabardAddictTab4:Hide()
	
	-- update portrait emblem
	SetPortraitToTexture(self.taTabardEmblem, "Interface\\ICONS\\INV_Chest_Cloth_30")
	
	--self.taTitleText:SetText(L.ADDON_TITLE)
	self.TitleContainer.TitleText:SetText(L.ADDON_TITLE)
	TabardAddictTab3.taVersionText:SetText("v"..TABARD_ADDICT_VERSION)

	-- set addon text
	self.Tab1Label:SetText(L.TAB1_LABEL)
	self.Tab2Label:SetText(L.TAB2_LABEL)
	self.Tab3Label:SetText(L.TAB3_LABEL)
	self.Tab4Label:SetText(L.TAB4_LABEL)

	-- config tab
	TabardAddictTab2.taConfigDescText:SetText(L.CONFIG_TITLE)
	TabardAddictTab2.fsTabardTypes:SetText(L.CONFIG_HDR1)
	TabardAddictTab2.fsTabardFilter:SetText(L.CONFIG_HDR2)
	TabardAddictTab2.fsTabardSugg:SetText(L.CONFIG_HDR3)
	TabardAddictTab2.ConfigExpansionsButton:SetText(L.CONFIG_PICK_EXPANSIONS)
	CheckButtonAO.fs:SetText(L.CONFIG_AO)
	CheckButtonHO.fs:SetText(L.CONFIG_HO)
	CheckButtonSuggAch.fs:SetText(L.CONFIG_SUGG_ACH)
	CheckButtonSuggQst.fs:SetText(L.CONFIG_SUGG_QST)
	CheckButtonSuggCur.fs:SetText(L.CONFIG_SUGG_CUR)
	CheckButtonTCG.fs:SetText(L.CONFIG_TCG)
	CheckButtonPVP.fs:SetText(L.CONFIG_PVP)
	CheckButtonNA.fs:SetText(L.CONFIG_NA)
	CheckButtonOther.fs:SetText(L.CONFIG_OTHER)

	TabardAddictTab3.taAboutDescText:SetText(L.ABOUT_TITLE)
	TabardAddictTab3.hdr1:SetText(L.ABOUT_HDR1)
	TabardAddictAboutInfo:SetText(L.ABOUT_TXT)
	
	TabardAddictTab4.taSuggestDescText:SetText(L.SUGGESTIONS_TITLE)
	
	-- Check Alliance/Horde/Neutral
	TabardAddict_CheckPlayerFaction()
	
	-- init name filter
	taNameFilter = ""
	TabardAddictTab1.SearchBox.label:SetText(SEARCH)

	-- make closable with escape
	tinsert(UISpecialFrames,self:GetName())

	-- TEMP ROW COUNT
	if (false) then
		local count = 0
		for _ in pairs(taTabardData) do count = count + 1 end
		print ("taTabardData rows: " .. count)
	end

end


